# CAQE - Clausal Abstraction for Quantifier Elimination

CAQE is a certifying solver for quantified Boolean formulas (QBF) in prenex conjunctive normal form (PNCF).
It is based on a recursive counterexample guided abstraction refinement (CEGAR) algorithm.

## Building

```
$ ./configure && make
```

## Solving

CAQE accepts formulas in the standard [QDIMACS](http://www.qbflib.org/qdimacs.html) file format.

`$ ./caqe instance.qdimacs` returns eiter SAT or UNSAT

### Parallelization

The parallel version of CAQE (pCAQE) can be enabled by `./configure --variant=parallel`


## Certification

### Format

Certificates are given as valid [AIGER](http://fmv.jku.at/aiger/) files with some additional conventions:

* The symboltable defines the original variable names from the QDIMACS file
* There is one additional output which must be the last output and it indicates whether the certificate is a Skolem or Herbrand certificate (value 1 and 0, respectively).

#### Example

[QDIMACS](http://www.qbflib.org/qdimacs.html) file encoding an equality constraint:

```
c equality.qdimacs
p cnf 2 2
a 1 0
e 2 0
-1 2 0
1 -2 0
```

The corresponding Skolem certificate is

```
aag 1 1 0 2 0
2
2
1
i0 1
o0 2
o1 result
c
equality.cert.aag
```

### Checking Certificates

* Prerequisites: `certcheck` binary (`$ make certification`)
* Usage: `$ ./certcheck <instance.qdimacs> <certificate>`
* Output of `certcheck` is an AIGER file that contains exactly one (bad) output.
* The output can be verified by a SAT solver: converting using `aigtocnf` from AIGER toolset and solving the resulting DIMACS file using a standard SAT solver.
* An UNSAT result witnesses the correctness of the certificate (there is no input such that the bad output is enabled).


#### Example

`$ ./certcheck equality.qdimacs equality.cert.aag | aigtocnf | picosat` returns unsatisfiable, hence, the certificate is correct


## Contact

Leander Tentrup <tentrup@react-uni-saarland.de>
