//
//  certification.h
//  caqe-qcir
//
//  Copyright (c) 2016, Leander Tentrup, Saarland University
//
//  Licensed under Apache License v2.0, see LICENSE.txt for information
//

#ifndef certification_h
#define certification_h

#include "aiger.h"
#include "matrix.h"
//#include "circuit_abstraction.h"
#include "queue.h"
#include "map.h"

typedef enum {
    SKOLEM,
    HERBRAND
} certificate_t;

typedef struct {
    aiger* skolem;
    aiger* herbrand;
    
    unsigned next_lit_skolem;
    unsigned next_lit_herbrand;
    
    int_queue queue;
    
    // incremental function building
    map* function_lit;      // maps var_id to current function literal
    map* precondition_lit;  // maps var_id to last precondition
} certification;

typedef struct ClausalAbstraction ClausalAbstraction;


void certification_init(certification*, Matrix*);

void certification_import_variables(certification*, Matrix*);

void certification_add_literal(certification*, lit_t);
void certification_add_b_literal(certification*, ClausalAbstraction*, quant_t, var_t);
void certification_add_t_literal(certification*, ClausalAbstraction*, quant_t, var_t, lit_t b_lit, lit_t t_lit);
void certification_define_b_literal(certification*, ClausalAbstraction*, quant_t, var_t);
bool certification_queue_is_empty(certification*);
/*unsigned certification_define_or(certification*, quant_t);
unsigned certification_define_and(certification*, quant_t);*/
void certification_add_aiger_literal(certification*, lit_t);
void certification_append_function_case(certification*, ClausalAbstraction*);

void certification_define_outputs(certification*, ClausalAbstraction*, qbf_res);
void certification_print(certification*, qbf_res);

#endif /* certification_h */
