//
//  clausal_abstraction.h
//  caqe
//
//  Copyright (c) 2015 - 2016, Leander Tentrup, Saarland University
//                2015, Markus Rabe, University of California, Berkeley
//
//  Licensed under Apache License v2.0, see LICENSE.txt for information
//

#ifndef __caqe__clausal_abstraction__
#define __caqe__clausal_abstraction__

#include <stdio.h>
#include "satsolver.h"
#include "map.h"
#include "solver.h"
#include "matrix.h"
#include "statistics.h"



typedef struct ClausalAbstraction ClausalAbstraction;

struct ClausalAbstraction {
    SATSolver* sat;
    SATSolver* negation;
    qbf_res result;
    
    int_vector* t_lits;
    int_vector* b_lits;
    int_vector* entry;
    int_vector* min_entry;
    int_vector* assumptions;
    int_vector* local_refinement_literals;
    
    // strong unsat refinement
    map* t_lit_mapping;
    var_t aux_var;
    int_vector* refinement;
    int_vector* conjunctive_lits;
    
    Solver* solver;
    Scope* scope;
    ClausalAbstraction** next;
    ClausalAbstraction* prev;

    Stats* statistics;
    
#ifdef PARALLEL_SOLVING
    pthread_mutex_t mutex;
    pthread_t thread;
    semaphore has_entry;
    semaphore sub_finished;
    size_t num_started;
#endif
};

typedef struct Solver Solver;

int clause_to_t_lit(Solver* solver, Clause* clause);
int clause_to_b_lit(Solver* solver, Clause* clause);
int b_lit_to_t_lit(Solver* solver, int b_lit);
int b_lit_to_clause_id(Solver* solver, int b_lit);
int t_lit_to_b_lit(Solver* solver, int t_lit);
int t_lit_to_clause_id(Solver* solver, int t_lit);

ClausalAbstraction* clausal_abstractions_init(Solver* solver, Scope* scope, ClausalAbstraction* prev);

#endif /* defined(__caqe__clausal_abstraction__) */

