//
//  logging.c
//  caqe
//
//  Copyright (c) 2015 - 2016, Leander Tentrup, Saarland University
//
//  Licensed under Apache License v2.0, see LICENSE.txt for information
//

#include "logging.h"

#include <stdio.h>
#include <stdarg.h>
#include <stdlib.h>

#define print_arguments(format, type) \
    va_list arg;\
    va_start(arg, format);\
    fprintf(stderr, type);\
    vfprintf(stderr, format, arg);\
    va_end(arg);

#define print_arguments_no_type(format) \
    va_list arg;\
    va_start(arg, format);\
    vfprintf(stderr, format, arg);\
    va_end(arg);

verbosity logging_verbosity = VERBOSITY_NORMAL;

#ifdef LOGGING

void logging_info(const char* format, ...) {
    if (logging_verbosity >= VERBOSITY_HIGH) {
        print_arguments_no_type(format);
    }
}

void logging_debug(const char* format, ...) {
    if (logging_verbosity >= VERBOSITY_ALL) {
        print_arguments_no_type(format);
    }
}

#endif

void logging_fatal(const char* format, ...) {
    print_arguments(format, "FATAL ERROR: ");
    abort();
}

void logging_error(const char* format, ...) {
    print_arguments(format, "error: ");
}

void logging_warn(const char* format, ...) {
    if (logging_verbosity >= VERBOSITY_NORMAL) {
        print_arguments(format, "warning: ");
    }
}

void logging_print(const char* format, ...) {
    if (logging_verbosity >= VERBOSITY_NORMAL) {
        print_arguments_no_type(format);
    }
}

#ifndef NO_API_ASSERTIONS
void api_expect(bool assertion, const char* format, ...) {
    if (!assertion) {
        print_arguments(format, "API error: ");
        abort();
    }
}
#endif

#ifndef NDEBUG
void fixme(const char* format, ...) {
    print_arguments(format, "fixme: ");
}
#endif

void logging_set_verbosity(verbosity v) {
    logging_verbosity = v;
}

#ifdef LOGGING
verbosity logging_get_verbosity() {
    return logging_verbosity;
}
#endif
