//
//  matrix_print.c
//  caqe
//
//  Copyright (c) 2016, Leander Tentrup, Saarland University
//
//  Licensed under Apache License v2.0, see LICENSE.txt for information
//

#include "matrix_print.h"

#include <assert.h>


static void print_scopes_with_id(Matrix* matrix, Scope* scope, var_t scope_id) {
    for (size_t i = 0; i < scope->num_next; i++) {
        Scope* next = scope->next[i];
        print_scopes_with_id(matrix, next, scope_id);
    }
    if (scope->scope_id != scope_id) {
        return;
    }
    if (vector_count(scope->vars) == 0) {
        return;
    }
    
    printf("%c ", scope->qtype == QUANTIFIER_EXISTENTIAL ? 'e' : 'a');
    for (size_t i = 0; i < vector_count(scope->vars); i++) {
        Var* var = vector_get(scope->vars, i);
        printf("%d ", var->var_id);
    }
    printf("0\n");
}

void matrix_print_qdimacs(Matrix* matrix) {
    printf("p cnf %zu %zu\n", matrix->max_vars, matrix->num_clauses);
    for (var_t i = 0; i < matrix->last_scope_id; i++) {
        print_scopes_with_id(matrix, matrix->top_level, i);
    }
    
    for (Clause* clause = matrix->first_clause; clause != NULL; clause = clause->next) {
        for (Occ* occ = clause->first_occ; occ != NULL; occ = occ->next_in_clause) {
            printf("%d ", matrix_get_lit(occ));
        }
        printf("0\n");
    }
    printf("\n");
}

void matrix_print_clause(Matrix* matrix, Clause* clause) {
    assert(matrix);
    for (Occ* occ = clause->first_occ; occ != NULL; occ = occ->next_in_clause) {
        printf("%d ", matrix_get_lit(occ));
    }
    printf("0\n");
}

static void print_prenex_recursive(Matrix* matrix, Scope* scope) {
    for (size_t i = 0; i < scope->num_next; i++) {
        Scope* next = scope->next[i];
        print_prenex_recursive(matrix, next);
        printf("\ts%p -> s%p;\n", (void*)scope, (void*)next);
    }
    
    Clause* first_clause = scope->first_clause == NULL ? matrix->first_clause : scope->first_clause;
    Clause* last_clause = scope->last_clause == NULL ? matrix->last_clause : scope->last_clause;
    
    printf("\ts%p[label=\"%d\n%d-%d\",color=\"%s\"];\n", (void*)scope, scope->scope_id, first_clause->clause_id, last_clause->clause_id, scope->qtype == QUANTIFIER_UNIVERSAL ? "red" : "green");
}

void matrix_print_partitioning_dot(Matrix* matrix) {
    printf("digraph circuit {\n");
    print_prenex_recursive(matrix, matrix->top_level);
    printf("}\n");
}
