//
//  qdimacs2qdimacs.c
//  caqe
//
//  Copyright (c) 2016, Leander Tentrup, Saarland University
//
//  Licensed under Apache License v2.0, see LICENSE.txt for information
//

#include <stdio.h>
#include <stdbool.h>

#include "getopt.h"
#include "logging.h"
#include "matrix.h"
#include "matrix_print.h"
#include "qdimacs.h"

static void print_usage(const char* name) {
    printf("usage: %s [-h] [-p] [file]\n\n"
           "optional arguments:\n"
           "  -h, --help\t\tshow this help message and exit\n"
           "  -p, --preprocess\tapply preprocessing during conversion\n", name);
}

int main(int argc, char* const argv[]) {
    const char* file_name = NULL;
    FILE* file = NULL;
    bool preprocessing = false;
    
    // Handling of command line arguments
    //opterr = 0; // Disable error messages as we handle them ourself
    const char * ch;
    while ((ch = GETOPT(argc, argv)) != NULL) {
        GETOPT_SWITCH(ch) {
        GETOPT_OPT("-h"):
            print_usage(argv[0]);
            return 1;
            break;
        GETOPT_OPT("-p"):
        GETOPT_OPT("--preprocess"):
            preprocessing = true;
            break;
        GETOPT_OPT("-v"):
            logging_set_verbosity(VERBOSITY_ALL);
            break;
        GETOPT_MISSING_ARG:
            printf("missing argument to %s\n", ch);
            /* FALLTHROUGH */
        GETOPT_DEFAULT:
            printf("unknown argument %s\n", ch);
            print_usage(argv[0]);
            return 1;
        }
    }
    
    if (optind + 1 < argc) {
        logging_error("too many arguments, see --help for more information\n");
        return 1;
    }
    
    if (optind < argc) {
        file_name = argv[optind];
    } else {
        file = stdin;
    }
    
    Matrix* matrix = matrix_init();
    if (!matrix) {
        return 1;
    }
    
    int error = 0;
    if (file == NULL) {
        error = qdimacs_open_and_read_qcir_file(matrix, file_name);
    } else {
        error = matrix_from_qdimacs(matrix, file);
    }
    if (error) {
        return 1;
    }
    
    if (preprocessing) {
        matrix_preprocess(matrix);
    }
    if (!matrix->conflict) {
        //matrix_print_qdimacs(matrix);
        //matrix_unprenex_by_miniscoping(matrix);
    }
    
    matrix_print_qdimacs(matrix);
    
    return 0;
}
