//
//  satsolver.h
//  caqe
//
//  Copyright (c) 2015 - 2016, Leander Tentrup, Saarland University
//
//  Licensed under Apache License v2.0, see LICENSE.txt for information
//

#ifndef SATSOLVER_H
#define SATSOLVER_H

#define SATSOLVER_SATISFIABLE 10
#define SATSOLVER_UNSATISFIABLE 20
#define SATSOLVER_UNKNOWN 0

#include <stdbool.h>
#include "vector.h"

struct SATSolver;
typedef struct SATSolver SATSolver;

typedef enum {
    SATSOLVER_RESULT_UNKNOWN = SATSOLVER_UNKNOWN,
    SATSOLVER_RESULT_SAT     = SATSOLVER_SATISFIABLE,
    SATSOLVER_RESULT_UNSAT   = SATSOLVER_UNSATISFIABLE
} sat_res;

SATSolver* satsolver_init(void);
void satsolver_free(SATSolver*);
void satsolver_adjust(SATSolver* solver, int variables);
void satsolver_save_original_clauses(SATSolver* solver);
void satsolver_reset(SATSolver* solver);

void satsolver_add(SATSolver* solver, int lit);
void satsolver_assume(SATSolver* solver, int lit);
void satsolver_clear_assumptions(SATSolver* solver);
bool satsolver_inconsistent(SATSolver* solver);
sat_res satsolver_sat(SATSolver* solver);
int satsolver_deref(SATSolver* solver, int lit);
int satsolver_deref_partial(SATSolver* solver, int lit);
int satsolver_deref_toplevel(SATSolver* solver, int lit);
int satsolver_is_decision_var(SATSolver* solver, int lit);
const int* satsolver_failed_assumptions(SATSolver* solver);
const int* satsolver_next_maximal_satisfiable_subset_of_assumptions(SATSolver* solver);

void satsolver_set_global_default_phase(SATSolver* s, int phase);
void satsolver_set_default_phase_lit(SATSolver* s, int lit, int phase);

void satsolver_set_more_important_lit(SATSolver* solver, int lit);
void satsolver_set_less_important_lit(SATSolver* solver, int lit);

double satsolver_seconds(SATSolver* solver);
void satsolver_print(SATSolver* solver);

const char* satsolver_get_name(void);
const char* satsolver_get_version(void);
const char* satsolver_copyright(void);

#endif
