//
//  satsolver_picosat.c
//  caqe
//
//  Copyright (c) 2015 - 2016, Leander Tentrup, Saarland University
//
//  Licensed under Apache License v2.0, see LICENSE.txt for information
//

#include "satsolver.h"

#include <assert.h>
#include <stdbool.h>
#include <stdint.h>
#include "picosat.h"

// Sanity check, make sure the return values are cotrrect
#if (PICOSAT_SATISFIABLE != SATSOLVER_SATISFIABLE) || (PICOSAT_UNSATISFIABLE != SATSOLVER_UNSATISFIABLE) || (PICOSAT_UNKNOWN != SATSOLVER_UNKNOWN)
#error "Return values of SAT solver and the generic SAT solver interface mismatch"
#endif


#define PICOSAT_DECISION_LIMIT -1

struct SATSolver {
    PicoSAT* instance;
};

SATSolver* satsolver_init() {
    SATSolver* solver = malloc(sizeof(SATSolver));
    solver->instance = picosat_init();
    return solver;
}

void satsolver_free(SATSolver* solver) {
    picosat_reset(solver->instance);
    free(solver);
}

void satsolver_adjust(SATSolver* solver, int variables) {
    picosat_adjust(solver->instance, variables);
}

void satsolver_save_original_clauses(SATSolver* solver) {
    picosat_save_original_clauses(solver->instance);
}

void satsolver_reset(SATSolver* solver) {
    picosat_reset(solver->instance);
    free(solver);
}

void satsolver_add(SATSolver* solver, int lit) {
    picosat_add(solver->instance, lit);
}

void satsolver_assume(SATSolver* solver, int lit) {
    picosat_assume(solver->instance, lit);
}

void satsolver_clear_assumptions(SATSolver* solver) {
    picosat_sat(solver->instance, 0);
}

bool satsolver_inconsistent(SATSolver* solver) {
    return picosat_inconsistent(solver->instance);
}

sat_res satsolver_sat(SATSolver* solver) {
    return picosat_sat(solver->instance, PICOSAT_DECISION_LIMIT);
}

int satsolver_deref(SATSolver* solver, int lit) {
    return picosat_deref(solver->instance, lit);
}

int satsolver_deref_partial(SATSolver* solver, int lit) {
    return picosat_deref_partial(solver->instance, lit);
}

int satsolver_deref_toplevel(SATSolver* solver, int lit) {
    return picosat_deref_toplevel(solver->instance, lit);
}

const int* satsolver_failed_assumptions(SATSolver* solver) {
    return picosat_failed_assumptions(solver->instance);;
}

void satsolver_push(SATSolver* solver) {
    picosat_push(solver->instance);
}

void satsolver_pop(SATSolver* solver) {
    picosat_pop(solver->instance);
}

double satsolver_seconds(SATSolver* solver) {
    return picosat_seconds(solver->instance);
}

void satsolver_set_global_default_phase(SATSolver* solver, int phase) {
    picosat_set_global_default_phase(solver->instance, phase);
}

void satsolver_set_default_phase_lit(SATSolver* solver, int lit, int phase) {
    picosat_set_default_phase_lit(solver->instance, lit, phase);
}

void satsolver_set_more_important_lit(SATSolver* solver, int lit) {
    picosat_set_more_important_lit(solver->instance, lit);
}
void satsolver_set_less_important_lit(SATSolver* solver, int lit) {
    picosat_set_less_important_lit(solver->instance, lit);
}

void satsolver_print(SATSolver* solver) {
    picosat_print(solver->instance, stdout);
}

const char* satsolver_get_name() {
    return "PicoSAT";
}

const char* satsolver_get_version() {
    return "965";
}

const char* satsolver_copyright() {
    return picosat_copyright();
}
