//
//  solver.h
//  caqe
//
//  Copyright (c) 2015 - 2016, Leander Tentrup, Saarland University
//                2015, Markus Rabe, University of California, Berkeley
//
//  Licensed under Apache License v2.0, see LICENSE.txt for information
//

#ifndef __caqe__solver__
#define __caqe__solver__

#include <stdbool.h>

#include "matrix.h"
#include "statistics.h"

#ifdef PARALLEL_SOLVING
#include "semaphore.h"
#endif

#ifdef CERTIFICATION
#include "certification.h"
#endif

typedef struct ClausalAbstraction ClausalAbstraction;

typedef struct {
    // high level features
    bool preprocess;
    bool miniscoping;
    bool certify;
    bool statistics;
    
    // low level solver features
    bool dual_propagation_minimization;
    bool strong_unsat_refinement;
    bool assume_partial_results;
    
#ifdef PARALLEL_SOLVING
    size_t num_threads;
#endif
} SolverOptions;


struct Solver {
    SolverOptions* options;
    Matrix* matrix;
    ClausalAbstraction* abstraction;
    
#ifdef PARALLEL_SOLVING
    semaphore num_threads;
#endif
    
#ifdef CERTIFICATION
    certification certificate;
#endif

    // statistics
    Stats* preprocessing;
    Stats* building_abstraction;
    Stats* solving;
};

typedef struct Solver Solver;

SolverOptions* solver_get_default_options(void);

Solver* solver_init(Matrix*, SolverOptions*);
void solver_free(Solver*);
qbf_res solver_sat(Solver*);
const char* solver_get_version(void);
void solver_print_statistics(Solver*);

#endif /* defined(__caqe__solver__) */
